package art.soft.level;

import art.soft.Loader;
import static art.soft.Settings.NUM_PLAYERS;
import art.soft.gameObjs.GameKeyListener;
import art.soft.gameObjs.gameObj;
import art.soft.objsData.ObjData;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;

/**
 *
 * @author Артём Святоха
 */
public class gameEngine extends gameObj implements GameKeyListener {

    private final static String START_LEVEL = "start";

    private final static String LEVEL_PATH = "levels/";

    private String currentLevel, restartLevel, deadLevel, nextLevel;

    public ObjData[] objects;

    public Layer layers, curLayer;

    public Layer[] layersArray;
    
    public int camX, camY; // Текущие координаты камеры

            // Переменные среды
    public float gravity; // Сила притяжения
    public boolean flipGravity; // Гравитация действует в обратном направлении
                    // при этом все динамические обьекты переварачивются
    public float envirFrictionX; // сопративление среды по оси X
    public float envirFrictionY; // сопративление среды по оси Y
            //

    public int numPlayers;

    public Player players[];
    
    public Camera cameras;
    
    public Camera curCamera;
    
    public Loader loader;

    private HashMap<String, ObjData> objs = new HashMap<>();

    public ObjData getObj(String objName) {
        return objs.get(objName);
    }

    private void readLevel(String name) {
        DataInputStream dis = loader.openFile(LEVEL_PATH + name + ".lev");
        int i, n;
        try {
            restartLevel = dis.readUTF();
            deadLevel = dis.readUTF();
            nextLevel = dis.readUTF();
            // Загрузка данных обьектов
            HashMap<String, ObjData> oldObjs = objs;
            objs = new HashMap<>();
            //
            n = dis.readUnsignedShort();
            objects = new ObjData[n];
            ObjData obj;
            for (i = 0; i < n; i ++) {
                name = dis.readUTF();
                obj = oldObjs.get(name);
                if (obj == null) {
                    obj = ObjData.loadObj(loader, name);
                } else {
                    oldObjs.remove(name);
                }
                objects[i] = obj;
                objs.put(name, obj);
            }
            oldObjs.forEach((nameO, objD) -> objD.remove());
            oldObjs.clear();
            // Загрузка слоёв
            Layer layerEnd = null;
            Layer layer;
            n = dis.readUnsignedByte();
            layersArray = new Layer[n];
            for (i = 0; i < n; i ++) {
                layer = loader.getObj(Layer.class);
                layer.readLayer(dis, loader);
                layersArray[i] = layer;
                //
                if (layerEnd == null) {
                    layerEnd = layers = layer;
                } else {
                    layerEnd.next = layer;
                    layerEnd = layer;
                }
            }
            // Загрузка камер
            Camera lastCam = null, camera;
            n = dis.readUnsignedShort();
            for (i = n - 1; i >= 0; i --) {
                camera = new Camera();
                camera.readCamera(dis, loader);
                //
                if (lastCam == null) {
                    lastCam = cameras = camera;
                } else {
                    lastCam.next = camera;
                    lastCam = camera;
                }
            }
            // Загрузка игроков
            n = dis.readUnsignedByte();
            Player[] oldPlayers = players;
            players = new Player[n];
            Player player;
            for (i = 0; i < n; i ++) {
                player = new Player();
                player.readPlayer(dis, loader);
                if (i < numPlayers || i > NUM_PLAYERS) {
                    if (i < numPlayers) {
                        player.active = true;
                        if (oldPlayers != null) {
                            player.item = oldPlayers[i].item;
                        }
                    }
                    player.init(loader, i < NUM_PLAYERS ?
                            loader.settings.playerKeys[i] : null);
                }
                players[i] = player;
            }
            dis.close();
        } catch (IOException ex) {
            loader.game.log("Cann't load level :\n" + ex.getMessage());
        }
    }

    public void loadLevel(String name) {
        int i;
        loader.level.removeAll();
        if (name.equals(currentLevel)) {
            Layer layer = layers;
            while (layer != null) {
                layer.init(loader);
                layer = (Layer) layer.next;
            }
            //
            Camera camera = cameras;
            while (camera != null) {
                camera.init(loader);
                camera = camera.next;
            }
            //
            if (players != null) {
                for (i = 0; i < numPlayers; i ++) {
                    players[i].active = true;
                    players[i].init(loader, loader.settings.playerKeys[i]);
                }
                for (; i < NUM_PLAYERS; i ++) {
                    players[i].active = false;
                }
                for (i = NUM_PLAYERS; i < players.length; i ++) {
                    players[i].init(loader, null);
                }
            }
        } else {
            if (players != null) {
                for (i = players.length - 1; i >= 0; i --) {
                    players[i].remove(loader);
                }
            }
            loader.poolAllObj(layers);
            layers = null;
            cameras = null;
            //
            currentLevel = name;
            readLevel(name);
        }
        camX = camY = 0;
        setCamera(cameras);
        //
        System.gc();
    }

    public ObjData getObj(int objIndx) {
        if (objIndx != 0xFFFF) {
            return objects[objIndx];
        } else {
            return null;
        }
    }

    @Override
    public void init(Loader loader) {
        this.loader = loader;
        loadLevel(START_LEVEL);
        playersItemReset();
    }

    private void playersItemReset() {
        if (players != null) {
            for (Player player : players) {
                player.removeItem();
            }
        }
    }

    public void restartLevel() {
        loadLevel(restartLevel);
        playersItemReset();
    }


    public void nextLevel() {
        loadLevel(nextLevel);
    }

    public void restertAfterDead() {
        loadLevel(deadLevel);
        playersItemReset();
    }

    public void setCamera(Camera camera) {
        curCamera = camera;
        gravity = camera.gravity;
        flipGravity = camera.flipGravity;
        envirFrictionX = camera.envirFrictionX;
        envirFrictionY = camera.envirFrictionY;
    }

    @Override
    public boolean act() {
        //
        Layer layer = curLayer = layers;
        while (layer != null) {
            layer.act();
            curLayer = layer = (Layer) layer.next;
        }
        //
        int cenX = 0;
        int cenY = 0;
        int numP = 0;
        for (Player player : players) {
            if (player.active) {
                player.act();
                if (!player.enemy) {
                    cenX += player.unit.x;
                    cenY += player.unit.y;
                    numP ++;
                }
            }
        }
        cenX /= numP;
        cenY /= numP;
        //
        int deltaX = camX - cenX;
        if (deltaX < 0) deltaX = - deltaX;
        int deltaY = camY - cenY;
        if (deltaY < 0) deltaY = - deltaY;
        //
        int qw = (int) (loader.game.width * curCamera.moveX_border);
        int qh = (int) (loader.game.height * curCamera.moveY_border);
        curCamera.setCenter((deltaX >= qw) ? (cenX > camX ? cenX - qw : cenX + qw) : camX,
                (deltaY >= qh) ? (cenY > camY ? cenY - qh : cenY + qh) : camY);
        return false;
    }

    @Override
    public void draw(Graphics g) {
        g.setColor(curCamera.background);
        g.fillRect(0, 0, loader.game.width, loader.game.height);
        //
        Layer layer = layers;
        while (layer != null) {
            layer.draw(g);
            layer = (Layer) layer.next;
        }
        if (loader.debugCamera) {
            int x = camX - (loader.game.width >> 1);
            int y = camY - (loader.game.height >> 1);
            g.translate(- x, - y);
            g.setColor(Color.GREEN);
            Camera camera = cameras;
            while (camera != null) {
                camera.draw(g);
                camera = camera.next;
            }
            g.translate(x, y);
        }
    }

    @Override
    public void pool(Loader loader) {
        loader.poolAllObj(layers);
        layers = null;
    }

    @Override
    public void keyPressed(int keyCode) {
        //
        if (players != null) {
            for (int i = numPlayers - 1; i >= 0; i --) {
                players[i].keyPressed(keyCode);
            }
        }
    }

    @Override
    public void keyReleased(int keyCode) {
        if (players != null) {
            for (int i = numPlayers - 1; i >= 0; i --) {
                players[i].keyReleased(keyCode);
            }
        }
    }
}
